package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.impl.DoubleConstant;
import org.eclipse.jdt.internal.compiler.util.FloatUtil;
import org.eclipse.jdt.core.compiler.CharOperation;

public class DoubleLiteral extends NumberLiteral
{
    double value;
    
    public DoubleLiteral(char[] token, int s, int e) {
        super(token, s, e);
    }
    
    @Override
    public void computeConstant() {
        boolean containsUnderscores = CharOperation.indexOf('_', this.source) > 0;
        if (containsUnderscores) {
            this.source = CharOperation.remove(this.source, '_');
        }
        Double computedValue;
        try {
            computedValue = Double.valueOf(String.valueOf(this.source));
        }
        catch (NumberFormatException ex) {
            try {
                double v = FloatUtil.valueOfHexDoubleLiteral(this.source);
                if (v == Double.POSITIVE_INFINITY) {
                    return;
                }
                if (Double.isNaN(v)) {
                    return;
                }
                this.value = v;
                this.constant = DoubleConstant.fromValue(v);
            }
            catch (NumberFormatException ex2) {}
            return;
        }
        double doubleValue = (double)computedValue;
        if (doubleValue > Double.MAX_VALUE) {
            return;
        }
        Label_0268: {
            if (doubleValue < Double.MIN_VALUE) {
                boolean isHexaDecimal = false;
                for (int i = 0; i < this.source.length; ++i) {
                    switch (this.source[i]) {
                        case '.':
                        case '0': {
                            break;
                        }
                        case 'X':
                        case 'x': {
                            isHexaDecimal = true;
                            break;
                        }
                        case 'D':
                        case 'E':
                        case 'F':
                        case 'd':
                        case 'e':
                        case 'f': {
                            if (isHexaDecimal) {
                                return;
                            }
                        }
                        case 'P':
                        case 'p': {
                            break Label_0268;
                        }
                        default: {
                            return;
                        }
                    }
                }
            }
        }
        this.value = doubleValue;
        this.constant = DoubleConstant.fromValue(this.value);
        return;
    }
    
    @Override
    public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
        int pc = codeStream.position;
        if (valueRequired) {
            codeStream.generateConstant(this.constant, this.implicitConversion);
        }
        codeStream.recordPositionsFrom(pc, this.sourceStart);
    }
    
    @Override
    public TypeBinding literalType(BlockScope scope) {
        return TypeBinding.DOUBLE;
    }
    
    @Override
    public void traverse(ASTVisitor visitor, BlockScope scope) {
        visitor.visit(this, scope);
        visitor.endVisit(this, scope);
    }
}
